#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include "neo.h"

int neo_function_arpfind(neo_macaddr *outmac, char *hostname, char *router)
{
  neo_searchable s;
  int ret;
  char buff[1024], val[500], macbuff[500];
  char *ipaddr;
  int k, numints;

  /* translate the ip */
  neo_searchable_create_from_string(&s, hostname);
  
  if (neo_searchable_primary_is_macaddr(&s)) {
    printf("function_arpfind: You must supply an IP address or hostname\n");
    neo_searchable_free(&s);
    return(-1);
  }
  
  /* we want the ip addr if we have the host */
  if (neo_searchable_primary_is_hostname(&s)) {
    ret=neo_searchable_expand_hostip(&s);
    if (ret) {
      printf("function_arpfind: Could not find ip address for host %s\n", hostname);
      neo_searchable_free(&s);
      return(-1);
    }
  }
  
  ipaddr=neo_searchable_get_ipaddr(&s);
  
  /* get the number of interfaces interfaces.ifNumber.0 */
  ret=neo_snmp_request_by_variable(router, ".1.3.6.1.2.1.2.1.0", val);
  if (ret) {
    printf("function_arpfind: Could not get number of interfaces from %s\n", router);
    /* neo_error_print(neo_error); */
    neo_searchable_free(&s);
    return(-1);
  }
  numints=atoi(val);
  if (neo_global_is_debug(g)) printf("function_arpfind: found %i interfaces on %s\n", numints, router);

  /* look for the address on each interface */
  for(k=1; k<=numints; k++) {
    if (neo_global_is_debug(g)) printf("function_arpfind: checking interface %i on %s\n", k, router);
    /* sprintf(buff,".1.3.6.1.2.1.3.1.1.2.%i.1.%s", k, ipaddr); */
    /* ipNetToMediaPhysAddress */
    sprintf(buff, ".1.3.6.1.2.1.4.22.1.2.%i.%s", k, ipaddr);
    ret=neo_snmp_request_by_variable(router, buff, val);
    if (ret && (neo_error!=NEO_ERR_SNMP_NOVAR)) {
      printf("function_arpfind: Error while talking to %s\n", router);
      neo_searchable_free(&s);
      return(-1);
    } else if (ret && (neo_error==NEO_ERR_SNMP_NOVAR)) {
      /* just go to the next one */
    } else {
      ret=neo_macaddr_create_from_string(outmac, val);
      if (ret) {
	printf("function_arpfind: Internal error parsing response into mac address (%s)\n", val);
	neo_searchable_free(&s);
	return(-1);
      } else {
	neo_searchable_free(&s);
	neo_macaddr_to_string(outmac, macbuff);
	if (neo_global_is_debug(g)) printf("function_arpfind: found %s is %s on %s interface %i\n", ipaddr, macbuff, router, k);
	return(0);
      }
    }
  }
  /* printf("arpfind: %s can not find entry for %s\n", router, ipaddr); */
  neo_searchable_free(&s);
  return(-1);
}

int neo_function_get_uplink(neo_locationlist *ans, char *devicename)
{
  int ret;
  neo_searchable s;
  char val[100], buff[1024];
  neo_macaddr m;
  neo_device d;

  if (neo_global_is_debug(g) || neo_global_is_verbose(g)) printf("Looking for uplink on %s\n", devicename);
  
  /* get the next hop ip with .1.3.6.1.2.1.4.21.1.7.0.0.0.0 */
  ret=neo_snmp_request_by_variable(devicename, ".1.3.6.1.2.1.4.21.1.7.0.0.0.0", val);
  if (ret) {
    if (neo_error==NEO_ERR_TIMEOUT || neo_error==NEO_ERR_CONTACT) {
      printf("function_get_uplink: Could not contact device %s\n", devicename);
      return(-1);
    }
    if (neo_global_is_verbose(g) || neo_global_is_debug(g)) printf("function_get_uplink: could not find route for 0.0.0.0 on %s\n", devicename);
    /* for now we'll bail, but I have more ideas for later */
    return(-1);
    
    /* If we couldn't find 0.0.0.0 look for any next hop ip address */
  }

  /* try looking up the mac address of that ip address in the devices'
   * arp cache.
   */
  ret=neo_function_arpfind(&m, val, devicename);
  if (ret) {
    /* If that didn't work, we'll go ask the router itself */
    if (neo_global_is_verbose(g) || neo_global_is_debug(g)) printf("Device %s did not have arp entry for %s, reverting to router %s\n", devicename, val, val);
    ret=neo_function_arpfind(&m, val, val);
    if (ret) {
      /* if that didn't work we'll fail for now */
      printf("function_get_uplink: failed arpfind for %s\n", val);
      return(-1);
    }
  }
  neo_macaddr_to_string(&m, buff);

  /* now locate that address on the device */
  neo_searchable_create_from_string(&s, buff);
  neo_device_create_and_set_const(&d, g, devicename);
  if (neo_global_is_debug(g)) printf("function_get_uplink: doing locate\n");
  ret=neo_device_command_locate(&d, &s, ans, 1);
  neo_searchable_free(&s);

  return(0);
}


/* Set the ifindex for every port in the layout.
 */
void neo_function_set_layout_ifindex(neo_device *d, neo_layout *lo)
{
  int i, j, ifindex;
  neo_boardport bp;

  /* go through every port on the layout */
  if (!neo_layout_has_boards(lo)) {
    if (neo_global_is_debug(g)) printf("function_set_layout_ifindex: %s does not have boards.\n", neo_device_get_name(d));
    /* no boards */
      
    j=neo_layout_get_numports(lo);
    for (i=0; i<j; i++) {
      neo_boardport_create_from_port_int(&bp, i+1);
      ifindex=neo_device_get_ifindex_from_boardport(d, &bp);
      neo_boardport_free(&bp);
      neo_layout_set_port_ifindex(lo, i+1, ifindex);
      if (neo_global_is_debug(g)) {
	printf("function_set_layout_ifindex: setting ifindex %i for port %i on %s.\n",
	       ifindex,
	       i+1,
	       neo_device_get_name(d));
      }

    }
  } else {
    if (neo_global_is_debug(g)) printf("function_set_layout_ifindex: %s has boards.\n", neo_device_get_name(d));
    /* boards */

    /* NEED TO IMPLEMENT THIS */
    /* RESUME CODING HERE */

  }

}
