#include <stdio.h>
#include "neo.h"

/* Initialize an empty layout
 */
void neo_layout_create(neo_layout *lo)
{
  neo_list_create(&(lo->numports_on_board));
  lo->hasboards=0;
  lo->numboards=0;
  lo->numports=0;
  neo_aa_create(&(lo->ifindex_aa));
  neo_aa_create(&(lo->bridgeindex_aa));
}

/* Set the number of boards on the device and indicate
 * that the device does have boards
 */
void neo_layout_set_num_boards(neo_layout *lo, int n)
{
  if (n!=0) {
    lo->hasboards=1;
    lo->numboards=n;
  } else {
    lo->hasboards=0;
    lo->numboards=0;
  }
}

/* Set the number of ports on the device and indicate that
 * the device does not have boards
 */
void neo_layout_set_num_ports(neo_layout *lo, int n)
{
  lo->hasboards=0;
  lo->numboards=0;
  lo->numports=n;
}

/* set the ifindex for the given port.  If used on a device that has
 * boards, -1 is returned.  Otherwise 0 is returned
 */
int neo_layout_set_port_ifindex(neo_layout *lo, int port, int ifindex)
{
  neo_boardport bp;
  char *valstr;

  if (lo->hasboards) return(-1);
  
  valstr=malloc(10);
  sprintf(valstr, "%i", ifindex);

  neo_boardport_create_from_port_int(&bp, port);
  neo_aa_set(&(lo->ifindex_aa),
	     neo_boardport_to_string(&bp),
	     valstr);
  neo_boardport_free(&bp);
  return(0);
}

/* set the bridge port for the given port.  If used on a device that
 * has boards, -1 is returned.  Otherwise 0 is returned
 */
int neo_layout_set_port_bridgeindex(neo_layout *lo, int port, int bridgeindex)
{
  neo_boardport bp;
  char *valstr;

  if (lo->hasboards) return(-1);

  valstr=malloc(10);
  sprintf(valstr, "%i", bridgeindex);

  neo_boardport_create_from_port_int(&bp, port);
  neo_aa_set(&(lo->bridgeindex_aa),
	     neo_boardport_to_string(&bp),
	     valstr);
  neo_boardport_free(&bp);
  return(0);
}

/* get the ifindex for a port.  If used on a device that has boards -1
 * is returned.  If no ifindex is set, -1 is returned.
 */
int neo_layout_get_port_ifindex(neo_layout *lo, int port)
{
  neo_boardport bp;
  char *bpstr, *val;

  if (lo->hasboards) return(-1);

  neo_boardport_create_from_port_int(&bp, port);
  bpstr=neo_boardport_to_string(&bp);
  val=neo_aa_get_val(&(lo->ifindex_aa), bpstr);
  free(bpstr);
  if (val==NULL) return(-1);
  return(atoi(val));
}

/* get the bridgeport for a port.  If used on a device that has boards
 * -1 is returned.  If no ifindex is set, -1 is returned.
 */
int neo_layout_get_port_bridgeport(neo_layout *lo, int port)
{
  neo_boardport bp;
  char *bpstr, *val;

  if (lo->hasboards) return(-1);

  neo_boardport_create_from_port_int(&bp, port);
  bpstr=neo_boardport_to_string(&bp);
  val=neo_aa_get_val(&(lo->bridgeindex_aa), bpstr);
  free(bpstr);
  if (val==NULL) return(-1);
  return(atoi(val));
}

/* Return 1 if the layout has boards, 0 if not
 */
int neo_layout_has_boards(neo_layout *lo)
{
  if (lo->hasboards) return(1);
  return(0);
}

/* Set the number of ports on board number 'board' to be 'numports'
 */
int neo_layout_set_board_numports(neo_layout *lo, int board, int numports)
{
  int size, diff, i, index;
  int *foo;

  if (!neo_layout_has_boards(lo)) return(-1);
  index=board-1;

  /* If the list is too short, pad it out */
  size=neo_list_get_size(&(lo->numports_on_board));
  if (size < index) {
    diff=index-size;
    for (i=0; i<diff; i++) {
      foo=malloc(sizeof(int));
      *foo=0;
      neo_list_append_element(&(lo->numports_on_board), foo);
    }
  }
  foo=malloc(sizeof(int));
  *foo=numports;
  neo_list_append_element(&(lo->numports_on_board), foo);
  return(0);
}

/* return the number of ports.  Only to be used on layouts that do not
 * have boards.
 */
int neo_layout_get_numports(neo_layout *lo)
{
  return(lo->numports);
}

/* Return the number of boards.  Returns 0 if the device does not have
 * boards
 */
int neo_layout_get_numboards(neo_layout *lo)
{
  if (!lo->hasboards) return(0);
  return(lo->numboards);
}

/* Return the number of ports on board number 'board'
 */
int neo_layout_get_board_numports(neo_layout *lo, int board)
{
  int size, *ports, index;

  if (board==0) return(0);
  index=board-1;
  size=neo_list_get_size(&(lo->numports_on_board));
  if (size <= index) return(0);

  ports=neo_list_get_element(&(lo->numports_on_board), index);
  return(*ports);
}

/* set the ifindex for board 'board' port 'port' to be 'ifindex'.
 * Return -1 if the layout does not have boards, otherwise return 0.
 */
int neo_layout_set_boardport_ifindex(neo_layout *lo, int board, int port, int ifindex)
{
  neo_boardport bp;
  char *valstr;

  if (!lo->hasboards) return(-1);
  
  valstr=malloc(10);
  sprintf(valstr, "%i", ifindex);

  neo_boardport_create_from_bp_ints(&bp, board, port);
  neo_aa_set(&(lo->ifindex_aa),
	     neo_boardport_to_string(&bp),
	     valstr);
  neo_boardport_free(&bp);
  return(0);
}

/* set the bridgeindex for board 'board' port 'port' to be
 * 'bridgeindex'.  Return -1 if the layout does not have boards,
 * otherwise return 0.
 */
int neo_layout_set_boardport_bridgeindex(neo_layout *lo, int board, int port, int bridgeindex)
{
  neo_boardport bp;
  char *valstr;

  if (!lo->hasboards) return(-1);

  valstr=malloc(10);
  sprintf(valstr, "%i", bridgeindex);

  neo_boardport_create_from_port_int(&bp, port);
  neo_aa_set(&(lo->bridgeindex_aa),
	     neo_boardport_to_string(&bp),
	     valstr);
  neo_boardport_free(&bp);
  return(0);
}

/* Get the ifindex for board 'board' port 'port'.  If the layout does
 * not have boards return -1
 */
int neo_layout_get_boardport_ifindex(neo_layout *lo, int board, int port)
{
  neo_boardport bp;
  char *bpstr, *val;

  if (!lo->hasboards) return(-1);

  neo_boardport_create_from_bp_ints(&bp, board, port);
  bpstr=neo_boardport_to_string(&bp);
  val=neo_aa_get_val(&(lo->ifindex_aa), bpstr);
  free(bpstr);
  if (val==NULL) return(-1);
  return(atoi(val));
}

/* Get the bridgeport for board 'board' port 'port'.  If the layout
 * does not have boards return -1
 */
int neo_layout_get_boardport_bridgeindex(neo_layout *lo, int board, int port)
{
  neo_boardport bp;
  char *bpstr, *val;

  if (!lo->hasboards) return(-1);

  neo_boardport_create_from_bp_ints(&bp, board, port);
  bpstr=neo_boardport_to_string(&bp);
  val=neo_aa_get_val(&(lo->bridgeindex_aa), bpstr);
  free(bpstr);
  if (val==NULL) return(-1);
  return(atoi(val));
}

void neo_layout_free(neo_layout *lo)
{
  neo_list_free_all(&(lo->numports_on_board), &free);
  neo_aa_free(&(lo->ifindex_aa));
  neo_aa_free(&(lo->bridgeindex_aa));
}

/* Print a debugging representation of the layout to stdout
 */
void neo_layout_debugprint(neo_layout *lo)
{
  int i, j;
  
  if (!neo_layout_has_boards(lo)) {
    printf("Number of ports: %i\n", neo_layout_get_numports(lo));
  } else {
    printf("Number of boards: %i\n", neo_layout_get_numboards(lo));
    j=neo_layout_get_numboards(lo);
    for (i=0; i<j; i++) {
      printf("  Ports on board %i: %i\n", i+1, neo_layout_get_board_numports(lo, i+1));
    }
  }
}
